"""The class implementation for the `new_releases` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialNewReleasesIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `new_releases` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M23 12l-2.44-2.78.34-3.68-3.61-.82-1.89-3.18L12 3 8.6 1.54 6.71 4.72l-3.61.81.34 3.68L1 12l2.44 2.78-.34 3.69 3.61.82 1.89 3.18L12 21l3.4 1.46 1.89-3.18 3.61-.82-.34-3.68L23 12zm-10 5h-2v-2h2v2zm0-4h-2V7h2v6z"/></svg>'  # noqa



"""This module is for translation-mapping
utility implementations.
"""

import importlib
import os
import re
from types import ModuleType
from typing import Dict
from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Union

from apysc._lint_and_doc.docs_lang import Lang
from apysc._lint_and_doc.docstring_util import DOCSTRING_PATH_COMMENT_PATTERN
from apysc._lint_and_doc.document_text_split_util import BodyText
from apysc._lint_and_doc.document_text_split_util import CodeBlock
from apysc._lint_and_doc.document_text_split_util import Heading
from apysc._lint_and_doc.lint_and_doc_hash_util import HashType

MAPPING_CONST_NAME: str = "MAPPING"

SKIPPING_PATTERNS: List[Pattern] = [
    re.compile(pattern=DOCSTRING_PATH_COMMENT_PATTERN),
]

_INTERFACE_SIGNATURE_PATTERN: str = r"\*\*\[Interface signature\]\*\* .*?<hr>"

_CODE_BLOCK_IFRAME_PATTERN: str = r'<iframe src=\\"static.*?index\.html\\".*?></iframe>'

MAPPING_UNNECESSARY_PATTERNS: List[Pattern] = [
    re.compile(pattern=_INTERFACE_SIGNATURE_PATTERN),
    re.compile(pattern=_CODE_BLOCK_IFRAME_PATTERN),
    re.compile(pattern=_CODE_BLOCK_IFRAME_PATTERN.replace("\\", "")),
]

_SplittedVals = List[Union[Heading, BodyText, CodeBlock]]


def read_mapping_data(*, src_doc_file_path: str, lang: Lang) -> Dict[str, str]:
    """
    Read an already saved mapping data.

    Parameters
    ----------
    src_doc_file_path : str
        A target source document file path.
    lang : Lang
        A target translation language.

    Returns
    -------
    already_saved_mapping : dict
        An already saved mapping data dictionary.
    """
    from apysc._file import module_util

    mapping_module_path: str = get_mapping_module_path(
        src_doc_file_path=src_doc_file_path, lang=lang
    )
    if not os.path.isfile(mapping_module_path):
        return {}
    module: ModuleType = module_util.read_target_path_module(
        module_path=mapping_module_path
    )
    importlib.reload(module)
    if not hasattr(module, MAPPING_CONST_NAME):
        return {}
    already_saved_mapping: Dict[str, str] = getattr(module, MAPPING_CONST_NAME)
    return already_saved_mapping


def get_mapping_module_path(*, src_doc_file_path: str, lang: Lang) -> str:
    """
    Get a mapping data module path.

    Parameters
    ----------
    src_doc_file_path : str
        A target source document file path.
    lang : Lang
        A target translation language.

    Returns
    -------
    mapping_module_path : str
        A mapping data module path.
    """
    basename: str = os.path.basename(src_doc_file_path)
    basename = basename.replace(".md", ".py", 1)
    mapping_module_path: str = os.path.join(
        f"./apysc/_translation/{lang.value}/",
        basename,
    )
    return mapping_module_path


def convert_splitted_values_to_keys(*, splitted_values: _SplittedVals) -> List[str]:
    """
    Convert specified splitted values to dictionary's key
    strings.

    Parameters
    ----------
    splitted_values : List of Heading, BodyText and CodeBlock
        Target splitted values.

    Returns
    -------
    keys : list of str
        Converted strings.
    """
    keys: List[str] = []
    for splitted_value in splitted_values:
        is_body_text: bool = False
        key: str = ""
        if isinstance(splitted_value, Heading):
            key = splitted_value.overall_text
        if isinstance(splitted_value, BodyText):
            is_body_text = True
            key = splitted_value.text
        if isinstance(splitted_value, CodeBlock):
            key = splitted_value.overall_code_block
        key = escape_key_or_value(key_or_val=key)
        if not is_body_text:
            keys.append(key)
        else:
            _append_body_text_keys_to_list(key=key, keys=keys)
    return keys


def _append_body_text_keys_to_list(*, key: str, keys: List[str]) -> None:
    """
    Append document's body text keys to a specified key's list.

    Parameters
    ----------
    key : str
        A target key string. This interface splits its string
        if there are two consecutive line breaks.
    keys : list of str
        A key's list to append.
    """
    splitted_keys: List[str] = key.split("\\n\\n")
    for key_ in splitted_keys:
        if _key_is_api_docs_list(key_=key_):
            lines: List[str] = key_.split("\\n")
            keys.extend(lines)
        elif _key_is_api_docs_br_tags_list(key_=key_):
            _extend_keys_with_api_docs_br_tags_list(keys=keys, key_=key_)
        else:
            keys.append(key_)


def _extend_keys_with_api_docs_br_tags_list(*, keys: List[str], key_: str) -> None:
    """
    Extend a specified keys list with an API docs' break tags list.

    Parameters
    ----------
    keys : list of str
        A keys list to extend.
    key_ : str
        An API docs' break tags list string.
    """
    splitted_keys: List[str] = key_.split("<br> ・")
    for i, splitted_key in enumerate(splitted_keys):
        if i != 0:
            splitted_keys[i] = f"<br> ・{splitted_key}"
    keys.extend(splitted_keys)


def _key_is_api_docs_br_tags_list(*, key_: str) -> bool:
    """
    Get a boolean indicating whether a specified key's string
    is a list markdown with break tags.

    Parameters
    ----------
    key_ : str
        A target key string to check.

    Returns
    -------
    result : bool
        A boolean indicates whether a specified key's string
        is a markdown of a list with break tags or not.
    """
    if "<br> ・" in key_:
        return True
    return False


_API_DOCS_LIST_PATTERN: Pattern = re.compile(pattern=r"^- .+?\\n", flags=re.MULTILINE)


def _key_is_api_docs_list(*, key_: str) -> bool:
    """
    Get a boolean indicating whether a specified key's
    string is an API document's (parameters, returns, exceptions,
    references) list.

    Parameters
    ----------
    key_ : str
        A target key string to check.

    Returns
    -------
    result : bool
        A boolean indicates whether a specified key's string
        is a markdown of a list with break tags or not.
    """
    match: Optional[Match] = _API_DOCS_LIST_PATTERN.search(string=key_)
    if match is None:
        return False
    return True


def escape_key_or_value(*, key_or_val: str) -> str:
    """
    Escape a mapping key or value to save.

    Parameters
    ----------
    key_or_val : str
        A target key or value.

    Returns
    -------
    key_or_val : str
        An escaped key or value string.
    """
    key_or_val = remove_escaping_from_key_or_value(key_or_val=key_or_val)
    TMP_LINE_END_ESCAPE_STR: str = "___tmp_line_end___"
    key_or_val = re.sub(
        pattern=r"\\$\n",
        repl=TMP_LINE_END_ESCAPE_STR,
        string=key_or_val,
        flags=re.MULTILINE,
    )
    key_or_val = key_or_val.replace("\\", "\\\\")
    key_or_val = key_or_val.replace("'", "\\'")
    key_or_val = key_or_val.replace('"', '\\"')
    key_or_val = key_or_val.replace("\n", "\\n")
    key_or_val = key_or_val.replace("\\\\n", "\\n")
    key_or_val = re.sub(
        pattern=rf"{TMP_LINE_END_ESCAPE_STR}",
        repl=r"\\\\\\n",
        string=key_or_val,
        flags=re.MULTILINE,
    )
    return key_or_val


def remove_escaping_from_key_or_value(*, key_or_val: str) -> str:
    """
    Remove escaping characters from a specified key or value string.

    Parameters
    ----------
    key_or_val : str
        A target key or value string.

    Returns
    -------
    key_or_val : str
        A result key or value string.
    """
    key_or_val = key_or_val.replace("\\\\", "\\")
    key_or_val = key_or_val.replace("\\'", "'")
    key_or_val = key_or_val.replace('\\"', '"')
    key_or_val = key_or_val.replace("\\n", "\n")
    return key_or_val


def is_mapping_unnecessary_key(*, key: str) -> bool:
    """
    Get a boolean indicating whether a specified key
    is an unnecessary mapping pattern or not.

    Parameters
    ----------
    key : str
        A target key string to check.

    Returns
    -------
    result : bool
        This interface returns True if a specified key is
        an unnecessary mapping pattern.
    """
    for pattern in MAPPING_UNNECESSARY_PATTERNS:
        match: Optional[Match] = pattern.search(string=key)
        if match is not None:
            return True
    return False


def is_translation_skipping_key(*, key: str) -> bool:
    """
    Get a boolean indicating whether a specified
    key is a translation skipping pattern or not.

    Parameters
    ----------
    key : str
        A target key string to check.

    Returns
    -------
    result : bool
        This interface returns True if a specified key
        is a translation skipping pattern.
    """
    for pattern in SKIPPING_PATTERNS:
        match: Optional[Match] = pattern.search(string=key)
        if match is not None:
            return True
    return False


def get_translated_file_path_from_src_path(*, source_doc_path: str, lang: Lang) -> str:
    """
    Get a translated file path from a specified source
    document's file path.

    Parameters
    ----------
    source_doc_path : str
        A target document's file path.
    lang : Lang
        A target language.

    Returns
    -------
    translated_file_path : str
        A translated file path.
    """
    basename: str = os.path.basename(source_doc_path)
    basename = f"{lang.value}_{basename}"
    dir_path = os.path.dirname(source_doc_path)
    translated_file_path: str = os.path.join(dir_path, basename)
    return translated_file_path


def remove_empty_keys(*, keys: List[str]) -> List[str]:
    """
    Remove empty (or only spaces) keys (source text) from
    a specified list.

    Parameters
    ----------
    keys : list of str
        A target keys list.

    Returns
    -------
    result_keys : list of str
        A result keys list.
    """
    result_keys: List[str] = []
    for key in keys:
        if key.strip() == "":
            continue
        result_keys.append(key)
    return result_keys


def get_hash_type_from_lang(*, lang: Lang) -> HashType:
    """
    Get a hash type from a specified language type.

    Parameters
    ----------
    lang : Lang
        A target translation language.

    Returns
    -------
    hash_type : HashType
        A target hash type.

    Raises
    ------
    ValueError
        If there is no implementation of a specified language
        type's branch condition.
    """
    if lang == Lang.JP:
        return HashType.TRANSLATION_MAPPING_JP
    raise ValueError(
        "There is no implementation of a specified language type's "
        "branch condition. Please add a necessary branch condition: "
        f"{lang}"
    )



"""Class implementation for the animation_time mix-in.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._type.variable_name_mixin import VariableNameMixIn


class AnimationTimeMixIn(VariableNameMixIn):
    @final
    @add_debug_info_setting(module_name=__name__)
    def animation_time(self) -> Number:
        """
        Get an animation elapsed millisecond.

        Returns
        -------
        elapsed_time : Number
            An animation elapsed millisecond.

        References
        ----------
        - animation_time interface
            - https://simon-ritchie.github.io/apysc/en/animation_time.html

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        ...
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options["rectangle"]
        ...     animation_time: ap.Number = rectangle.animation_time()
        ...     ap.trace("animation_time:", animation_time)
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> _ = rectangle.animation_x(
        ...     x=100,
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        >>> options: RectOptions = {"rectangle": rectangle}
        >>> ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()
        """
        from apysc._expression import expression_data_util

        elapsed_time: Number = Number(0.0)
        expression: str = (
            f"{elapsed_time.variable_name} = "
            f"{self.variable_name}.timeline().time();"
        )
        expression_data_util.append_js_expression(expression=expression)
        return elapsed_time



"""The mix-in class implementation for the `SvgText`'s `_set_italic` method.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean


class SvgTextSetItalicMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_italic(
        self,
        *,
        italic: Optional[Union[bool, Boolean]],
    ) -> None:
        """
        Set an italic style setting.

        Parameters
        ----------
        italic : Optional[Union[bool, Boolean]]
            A boolean whether a text is in an italic style or not (normal).
            If a specified value is `None`, this interface ignores the setting.
        """
        from apysc._display.svg_text_italic_mixin import SvgTextItalicMixIn
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if italic is None:
            return

        if not isinstance(self, SvgTextItalicMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextItalicMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        if isinstance(italic, bool):
            suffix: str = get_attr_or_variable_name_suffix(
                instance=self,
                value_identifier="italic",
            )
            italic_: Boolean = Boolean(italic, variable_name_suffix=suffix)
        else:
            italic_ = italic
        self.italic = italic_



"""The mix-in class implementation for the `String`'s `length` method.
"""

from typing import TYPE_CHECKING

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._type.int import Int


class StringLengthMixIn:
    @property
    @arg_validation_decos.is_apysc_string(arg_position_index=0)
    @add_debug_info_setting(module_name=__name__)
    def length(self) -> "Int":
        """
        Get a character length (number).

        Returns
        -------
        characters_length : Int
            A character length (number).

        References
        ----------
        - String class length property
            - https://simon-ritchie.github.io/apysc/en/string_length.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> string: ap.String = ap.String("Hello")
        >>> string.length
        Int(5)
        """
        from apysc._expression import expression_data_util
        from apysc._type.int import Int
        from apysc._type.string import String
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        self_variable_name: str = validate_variable_name_mixin_type(
            instance=self,
        ).variable_name
        characters_length: Int = Int(0)
        if isinstance(self, String):
            characters_length._value = len(self._value)
        expression: str = (
            f"{characters_length.variable_name} = [...{self_variable_name}].length;"
        )
        expression_data_util.append_js_expression(expression=expression)
        return characters_length



"""The class implementation for the `desktop_access_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialDesktopAccessDisabledIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `desktop_access_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M23 16c0 1.1-.9 2-2 2h-1l-2-2h3V4H6L4 2h17c1.1 0 2 .9 2 2v12zm-5.5 2l-2-2zm-2.6 0l6 6 1.3-1.3-4.7-4.7-2-2L1.2 1.8 0 3.1l1 1V16c0 1.1.9 2 2 2h7v2H8v2h8v-2h-2v-2h.9zM3 16V6.1l9.9 9.9H3z"/></svg>'  # noqa



"""The class implementation for the outlined `mark_email_unread` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMarkEmailUnreadOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `mark_email_unread` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/><path d="M22,8.98V18c0,1.1-0.9,2-2,2H4c-1.1,0-2-0.9-2-2L2.01,6C2.01,4.9,2.9,4,4,4h10.1C14.04,4.32,14,4.66,14,5s0.04,0.68,0.1,1 H4l8,5l3.67-2.29c0.47,0.43,1.02,0.76,1.63,0.98L12,13L4,8v10h16V9.9C20.74,9.75,21.42,9.42,22,8.98z M16,5c0,1.66,1.34,3,3,3 s3-1.34,3-3s-1.34-3-3-3S16,3.34,16,5z"/></g></svg>'  # noqa



"""The class implementation for the `remove_done` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRemoveDoneIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `remove_done` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0zm0 0h24v24H0V0z" fill="none"/><path d="M1.79 12l5.58 5.59L5.96 19 .37 13.41 1.79 12zm.45-7.78L12.9 14.89l-1.28 1.28L7.44 12l-1.41 1.41L11.62 19l2.69-2.69 4.89 4.89 1.41-1.41L3.65 2.81 2.24 4.22zm14.9 9.27L23.62 7 22.2 5.59l-6.48 6.48 1.42 1.42zM17.96 7l-1.41-1.41-3.65 3.66 1.41 1.41L17.96 7z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: int_and_number_to_fixed.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Int and Number classes to_fixed interface": "# Int と Number クラスの to_fixed インターフェイス",  # noqa
    ##################################################
    "This page explains the `Int` and `Number` classes `to_fixed` method interface.": "このページでは`Int`と`Number`クラスの`to_fixed`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `to_fixed` method converts a number to a fixed floating-point string notation.": "`to_fixed`メソッドは数値を固定の桁数の浮動小数点数を持った文字列へと変換します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `to_fixed` method requires a `digits` argument.": "`to_fixed`メソッドは`digits`引数の指定を必要とします。",  # noqa
    ##################################################
    "Its argument accepts between 0 to 100 range value.": "その引数は0～100の範囲の値を受け付けます。",
    ##################################################
    "If you specify 0 to the `digits` argument, a result string becomes an integer string.": "もし`digits`引数に0を指定した場合、結果の文字列は整数の形式の文字列になります。",  # noqa
    ##################################################
    "Also, if you set 2 to the `digits` argument, a result string becomes a number with two decimal places string (e.g., 10.34).": "また、`digits`引数に2を指定した場合には結果の文字列は小数点以下が2桁の数値の文字列になります（例 : 10.34）。",  # noqa
    ##################################################
    "This interface reflects even rounding to a truncated floating-point number.": "このインターフェイスは切り捨てられた浮動小数点数に偶数丸めの処理を反映します。",  # noqa
    ##################################################
    "For example, if a number is 10.678, and a `digits` argument is 2, a result string becomes 10.68.": "例えばもし数値が10.678であり`digits`引数がに2が指定されたばあぽ、結果の文字列は10.68になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nnum: ap.Number = ap.Number(10.789)\nfixed_float_str: ap.String = num.to_fixed(digits=2)\nap.assert_equal(fixed_float_str, "10.79")\n\nfixed_float_str = num.to_fixed(digits=5)\nap.assert_equal(fixed_float_str, "10.78900")\n\nfixed_float_str = num.to_fixed(digits=0)\nap.assert_equal(fixed_float_str, "11")\n\nap.save_overall_html(dest_dir_path="to_fixed_basics_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nnum: ap.Number = ap.Number(10.789)\nfixed_float_str: ap.String = num.to_fixed(digits=2)\nap.assert_equal(fixed_float_str, "10.79")\n\nfixed_float_str = num.to_fixed(digits=5)\nap.assert_equal(fixed_float_str, "10.78900")\n\nfixed_float_str = num.to_fixed(digits=0)\nap.assert_equal(fixed_float_str, "11")\n\nap.save_overall_html(dest_dir_path="to_fixed_basics_usage/")\n```',  # noqa
    ##################################################
    "## to_fixed API": "## to_fixed API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Convert value to fixed floating point string notation.<hr>": "値を固定の小数点以下の桁数の文字列へと変換します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `digits`: int or Int": "- `digits`: int or Int",
    ##################################################
    "  - A floating point digit number (0 to 100 value is acceptable).": "  - 浮動小数点数の桁数（0～100の範囲の値を受け付けることができます）。",  # noqa
    ##################################################
    "- `variable_name_suffix`: str, optional": "- `variable_name_suffix`: str, optional",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result_str`: String": "- `result_str`: String",
    ##################################################
    "  - A converted string.": "  - 変換された文字列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> num: ap.Number = ap.Number(10.789)\n>>> fixed_float_str: ap.String = num.to_fixed(digits=2)\n>>> fixed_float_str\nString("10.79")\n\n>>> fixed_float_str = num.to_fixed(digits=5)\n>>> fixed_float_str\nString("10.78900")\n\n>>> fixed_float_str = num.to_fixed(digits=0)\n>>> fixed_float_str\nString("11")\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> num: ap.Number = ap.Number(10.789)\n>>> fixed_float_str: ap.String = num.to_fixed(digits=2)\n>>> fixed_float_str\nString("10.79")\n\n>>> fixed_float_str = num.to_fixed(digits=5)\n>>> fixed_float_str\nString("10.78900")\n\n>>> fixed_float_str = num.to_fixed(digits=0)\n>>> fixed_float_str\nString("11")\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: array.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class": "# Array クラス",
    ##################################################
    "This page explains the `Array` class.": "このページでは`Array`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page:": "事前に以下のページを確認しておくと読み進める上で役に立つかもしれません:",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the Array?": "## Array クラスとは",
    ##################################################
    "The `Array` class is the apysc array class. It behaves like the Python built-in `list` value.": "`Array`クラスはapyscの配列のクラスです。このクラスはPythonビルトインの`list`のように動作します。",  # noqa
    ##################################################
    "## Constructor method": "## コンストラクタメソッド",
    ##################################################
    "The `Array` class constructor method requires iterable objects, like the `list`\\, `tuple`\\, `range`\\, or `Array` value.": "`Array`クラスのコンストラクタでは`list`や`tuple`、`range`、`Array`などのイテラブルなオブジェクトが引数に必要となります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr_from_list: ap.Array = ap.Array([1, 2, 3])\nassert arr_from_list == [1, 2, 3]\n\narr_from_tuple: ap.Array = ap.Array((4, 5, 6))\nassert arr_from_tuple == [4, 5, 6]\n\nother_arr: ap.Array = ap.Array([7, 8, 9])\narr_from_arr: ap.Array = ap.Array(other_arr)\nassert arr_from_arr == [7, 8, 9]\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr_from_list: ap.Array = ap.Array([1, 2, 3])\nassert arr_from_list == [1, 2, 3]\n\narr_from_tuple: ap.Array = ap.Array((4, 5, 6))\nassert arr_from_tuple == [4, 5, 6]\n\nother_arr: ap.Array = ap.Array([7, 8, 9])\narr_from_arr: ap.Array = ap.Array(other_arr)\nassert arr_from_arr == [7, 8, 9]\n```",  # noqa
    ##################################################
    "## Generic type annotation": "## ジェネリックの型アノテーション",
    ##################################################
    "If the `Array` values types are unique, you can set the generic type to an `Array` value. This annotation may be helpful when you use it on the IDE (for type checkers).": "もし`Array`クラスの値の型が一意な場合は配列に対してジェネリックの型の指定を行うことができます。この型アノテーションはIDE上などで便利なケースがあります（型チェックのライブラリなどを使う場合に）。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 2])\nint_val: int = arr.pop()\nassert isinstance(int_val, int)\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 2])\nint_val: int = arr.pop()\nassert isinstance(int_val, int)\n```",  # noqa
    ##################################################
    "## fixed_value_type argument": "## fixed_value_type 引数について",
    ##################################################
    "The constructor's `fixed_value_type` argument is optional.": "コンストラクタの`fixed_value_type`引数は省略可能な引数になっています。",  # noqa
    ##################################################
    "If specified, the array value-related interfaces, such as the `__getitem__` method (array subscript interface, for example, `arr[5]`), becomes possibly returning a specified type instance.": "もし指定された場合、`__getitem__`メソッド（`arr[5]`などの配列の添え字のインターフェイスが該当します）などの配列の値に関係した各インターフェイスが指定された型のインスタンスを返却するケースが発生するようになります",  # noqa
    ##################################################
    "It returns an instance of the specified type, especially when an array index exceeds the length of an array on the Python runtime.": "この返却処理はPythonランタイム上で配列の添え字が配列の長さを越える場合などに発生するようになります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=100,\n    stage_height=50,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\narr: ap.Array[ap.Int] = ap.Array([ap.Int(10)], fixed_value_type=ap.Int)\n\n# Index 5 is out of bounds on the Python runtime,\n# but since the `fixed_value_type` is the `ap.Int`,\n# this interface returns an `ap.Int` value.\nint_value: ap.Int = arr[5]\nassert isinstance(int_value, ap.Int)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=100,\n    stage_height=50,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\narr: ap.Array[ap.Int] = ap.Array([ap.Int(10)], fixed_value_type=ap.Int)\n\n# Index 5 is out of bounds on the Python runtime,\n# but since the `fixed_value_type` is the `ap.Int`,\n# this interface returns an `ap.Int` value.\nint_value: ap.Int = arr[5]\nassert isinstance(int_value, ap.Int)\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Funcdamental data classes common value interface](fundamental_data_classes_value_interface.md)": "- [基本的なデータクラスの共通の value インターフェイス](jp_fundamental_data_classes_value_interface.md)",  # noqa
    ##################################################
    "- [Array class append and push interfaces](array_append_and_push.md)": "- [Array クラスの append と push のインターフェイス](jp_array_append_and_push.md)",  # noqa
    ##################################################
    "- [Array class extend and concat interfaces](array_extend_and_concat.md)": "- [Array クラスの extend と concat のインターフェイス](jp_array_extend_and_concat.md)",  # noqa
    ##################################################
    "- [Array class insert and insert_at interfaces](array_insert_and_insert_at.md)": "- [Array クラスの insert と insert_at のインターフェイス](jp_array_insert_and_insert_at.md)",  # noqa
    ##################################################
    "- [Array class pop interface](array_pop.md)": "- [Array クラスの pop インターフェイス](jp_array_pop.md)",  # noqa
    ##################################################
    "- [Array class remove and remove_at interfaces](array_remove_and_remove_at.md)": "- [Array クラスの remove と remove_at のインターフェイス](jp_array_remove_and_remove_at.md)",  # noqa
    ##################################################
    "- [Array class sort interface](array_sort.md)": "- [Array クラスの sort インターフェイス](jp_array_sort.md)",  # noqa
    ##################################################
    "- [Array class reverse interface](array_reverse.md)": "- [Array クラスの reverse インターフェイス](jp_array_reverse.md)",  # noqa
    ##################################################
    "- [Array class slice interface](array_slice.md)": "- [Array クラスの slice インターフェイス](jp_array_slice.md)",  # noqa
    ##################################################
    "- [Array class length interface](array_length.md)": "- [Array クラスの length (配列の長さ取得) のインターフェイス](jp_array_length.md)",  # noqa
    ##################################################
    "- [Array class join interface](array_join.md)": "- [Array クラスの join (値の連結文字列生成) のインターフェイス](jp_array_join.md)",  # noqa
    ##################################################
    "- [Array class index_of interface](array_index_of.md)": "- [Array クラスの index_of (値のインデックス取得) のインターフェイス](jp_array_index_of.md)",  # noqa
    ##################################################
    "- [Array class comparison interfaces](array_comparison.md)": "- [Array クラスの比較の各インターフェイス](jp_array_comparison.md)",  # noqa
    ##################################################
    "## Array class constructor API": "## Array クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Array class for the apysc library.<hr>": "apyscライブラリの配列を扱うためのクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: Array or list or tuple or range": "- `value`: Array or list or tuple or range",  # noqa
    ##################################################
    "  - Initial array value.": "  - 配列の初期値。",
    ##################################################
    "- `fixed_value_type`: Optional[Type[InitializeWithBaseValueInterface]], optional": "- `fixed_value_type`: Optional[Type[InitializeWithBaseValueInterface]], optional",  # noqa
    ##################################################
    "  - A fixed value type of array. This argument only becomes an apysc type, such as the `ap.Int`, `ap.String`, or `ap.Rectangle`. If specified, the array value-related interfaces, such as the `__getitem__` method (array subscript interface, for example, `arr[5]`), becomes possibly returning a specified value type instance.": "  - 配列の値の固定の型。この引数は`ap.Int`、`ap.String`、`ap.Rectangle`などのapyscの型のみ設定することができます。もし指定された場合、`__getitem__`などのメソッド（`arr[5]`などの配列の添え字のインターフェイスが該当します）が指定された型のインスタンスを返却するケースが発生するようになります。",  # noqa
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "- `skip_init_substitution_expression_appending`: bool, default False": "- `skip_init_substitution_expression_appending`: bool, default False",  # noqa
    ##################################################
    "  - A boolean indicates whether to skip an initial substitution expression or not. This class uses this option internally.": "  - 初期値の代入のコード表現をスキップするかどうかの真偽値です。このオプションはクラス内部の実装で使用されます。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr\nArray([1, 2, 3])\n\n>>> arr[0]\n1\n\n>>> arr[1]\n2\n\n>>> arr = ap.Array((4, 5, 6))\n>>> arr\nArray([4, 5, 6])\n\n>>> arr = ap.Array(range(3))\n>>> arr\nArray([0, 1, 2])\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr\nArray([1, 2, 3])\n\n>>> arr[0]\n1\n\n>>> arr[1]\n2\n\n>>> arr = ap.Array((4, 5, 6))\n>>> arr\nArray([4, 5, 6])\n\n>>> arr = ap.Array(range(3))\n>>> arr\nArray([0, 1, 2])\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Array class comparison interfaces](https://simon-ritchie.github.io/apysc/en/array_comparison.html)": "- [Array クラスの比較の各インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_array_comparison.html)",  # noqa
    ##################################################
    "## value property API": "## value 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current array value.<hr>": "現在の配列の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `value`: list": "- `value`: list",
    ##################################################
    "  - Current array value.": "  - 現在の配列の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.value = [4, 5, 6]\n>>> arr.value\n[4, 5, 6]\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.value = [4, 5, 6]\n>>> arr.value\n[4, 5, 6]\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [apysc fundamental data classes value interface](https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html)": "- [apyscの基本的なデータクラスの value インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_fundamental_data_classes_value_interface.html)",  # noqa
}



"""The class implementation for the `archive` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialArchiveIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `archive` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20.54 5.23l-1.39-1.68C18.88 3.21 18.47 3 18 3H6c-.47 0-.88.21-1.16.55L3.46 5.23C3.17 5.57 3 6.02 3 6.5V19c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V6.5c0-.48-.17-.93-.46-1.27zM12 17.5L6.5 12H10v-2h4v2h3.5L12 17.5zM5.12 5l.81-1h12l.94 1H5.12z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: to_string.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# to_string interface": "# to_string インターフェイス",
    ##################################################
    "This page explains the `to_string` method interface.": "このページでは`to_string`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `to_string` method returns a `String` representation of itself.": "`to_string`メソッドはそのインスタンス自体の`String`型での表現の値を返却します。",  # noqa
    ##################################################
    "This interface exists on each basic data class, such as the `Int`, `Number`,  `Boolean`, or `Array`.": "このインターフェイスは`Int`や`Number`、`Boolean`や`Array`などの基本的な各データクラスに存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `to_string` method requires no arguments.": "`to_string`メソッドは引数の指定を必要としません。",
    ##################################################
    "A result value becomes a JavaScript-based value.": "返却値はJavaScriptに準じた値となります。",
    ##################################################
    "For instance, a `Boolean` value becomes `true` or `false`, not `True` or `False`.": "たとえば、`Boolean`の値であれば`True`や`False`などの値ではなく`true`や`false`といった文字列の値になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=0,\n    stage_height=0,\n    stage_elem_id="stage",\n)\nint_value: ap.Int = ap.Int(100)\nstring: ap.String = int_value.to_string()\nap.assert_equal(string, "100")\n\nnumber_value: ap.Number = ap.Number(10.5)\nstring = number_value.to_string()\nap.assert_equal(string, "10.5")\n\nbool_value: ap.Boolean = ap.Boolean(True)\nstring = bool_value.to_string()\nap.assert_equal(string, "true")\n\narray_value: ap.Array = ap.Array([10, 20, 30])\nstring = array_value.to_string()\nap.assert_equal(string, "10,20,30")\n\nap.save_overall_html(dest_dir_path="to_string_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=0,\n    stage_height=0,\n    stage_elem_id="stage",\n)\nint_value: ap.Int = ap.Int(100)\nstring: ap.String = int_value.to_string()\nap.assert_equal(string, "100")\n\nnumber_value: ap.Number = ap.Number(10.5)\nstring = number_value.to_string()\nap.assert_equal(string, "10.5")\n\nbool_value: ap.Boolean = ap.Boolean(True)\nstring = bool_value.to_string()\nap.assert_equal(string, "true")\n\narray_value: ap.Array = ap.Array([10, 20, 30])\nstring = array_value.to_string()\nap.assert_equal(string, "10,20,30")\n\nap.save_overall_html(dest_dir_path="to_string_basic_usage_1/")\n```',  # noqa
    ##################################################
    "Sometimes this method becomes useful when using text-related interfaces with its string.": "このメソッドはその文字列を使ってテキスト関係のインターフェイスを使う際に便利なことがあります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=50,\n    stage_elem_id="stage",\n)\nwidth: ap.Int = ap.Int(50)\ntext: ap.SvgText = ap.SvgText(\n    text=ap.String("width is: ") + width.to_string(),\n    fill_color=ap.Color("#aaa"),\n    x=20,\n    y=30,\n)\nap.save_overall_html(dest_dir_path="to_string_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=300,\n    stage_height=50,\n    stage_elem_id="stage",\n)\nwidth: ap.Int = ap.Int(50)\ntext: ap.SvgText = ap.SvgText(\n    text=ap.String("width is: ") + width.to_string(),\n    fill_color=ap.Color("#aaa"),\n    x=20,\n    y=30,\n)\nap.save_overall_html(dest_dir_path="to_string_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## to_string method API": "## to_string メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Convert this instance to a string.<hr>": "このインスタンスを文字列へと変換します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `string`: String": "- `string`: String",
    ##################################################
    "  - A converted string.": "  - 変換された文字列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"), stage_width=200, stage_height=200\n... )\n>>> int_value: ap.Int = ap.Int(value=100)\n>>> string: ap.String = int_value.to_string()\n>>> ap.assert_equal(string, "100")\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"), stage_width=200, stage_height=200\n... )\n>>> int_value: ap.Int = ap.Int(value=100)\n>>> string: ap.String = int_value.to_string()\n>>> ap.assert_equal(string, "100")\n```',  # noqa
}



"""The class implementation for the outlined `privacy_tip` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPrivacyTipOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `privacy_tip` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M12,3.19l7,3.11V11c0,4.52-2.98,8.69-7,9.93C7.98,19.69,5,15.52,5,11V6.3L12,3.19 M12,1L3,5v6c0,5.55,3.84,10.74,9,12 c5.16-1.26,9-6.45,9-12V5L12,1L12,1z M11,7h2v2h-2V7z M11,11h2v6h-2V11z"/></g></svg>'  # noqa



"""The class implementation for the outlined `flaky` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFlakyOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `flaky` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M14.05,17.58l-0.01,0.01l-2.4-2.4l1.06-1.06l1.35,1.35L16.54,13l1.06,1.06 l-3.54,3.54L14.05,17.58z M12,2C6.5,2,2,6.5,2,12s4.5,10,10,10s10-4.5,10-10S17.5,2,12,2z M7.34,6.28l1.41,1.41l1.41-1.41 l1.06,1.06L9.81,8.75l1.41,1.41l-1.06,1.06L8.75,9.81l-1.41,1.41l-1.06-1.06l1.41-1.41L6.28,7.34L7.34,6.28z M12,20 c-2.2,0-4.2-0.9-5.7-2.3L17.7,6.3C19.1,7.8,20,9.8,20,12C20,16.4,16.4,20,12,20z" fill-rule="evenodd"/></g></svg>'  # noqa



"""The class implementation for the outlined `zoom_in` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialZoomInOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `zoom_in` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M15.5 14h-.79l-.28-.27C15.41 12.59 16 11.11 16 9.5 16 5.91 13.09 3 9.5 3S3 5.91 3 9.5 5.91 16 9.5 16c1.61 0 3.09-.59 4.23-1.57l.27.28v.79l5 4.99L20.49 19l-4.99-5zm-6 0C7.01 14 5 11.99 5 9.5S7.01 5 9.5 5 14 7.01 14 9.5 11.99 14 9.5 14zm.5-7H9v2H7v1h2v2h1v-2h2V9h-2z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: set_debug_mode.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# set_debug_mode interface": "# set_debug_mode インターフェイス",
    ##################################################
    "This page explains the `set_debug_mode` function interface.": "このページでは`set_debug_mode`関数のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `set_debug_mode` function interface sets the debug mode setting. This setting appends the debug information (Python function or method calls and arguments information) to the exported HTML.": "`set_debug_mode`関数のインターフェイスはデバッグモードの設定を有効化します。この設定はデバッグ用の情報（Python上の関数やメソッドの呼び出しと引数情報など）を出力されるHTML上に追加します。",  # noqa
    ##################################################
    "## Notes": "## 特記事項",
    ##################################################
    "The debug mode setting appends lots of information to the HTML. As a result, the exporting time becomes long, and the HTML file size becomes large.": "デバッグモードの設定は多くの情報をHTML上に追加します。結果として出力時間は長くなり、ファイルサイズも大きくなります。",  # noqa
    ##################################################
    "Also, this setting ignores the `minify` setting.": "また、この設定は`minify`（HTML最小化）の設定も無視します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "After the stage instantiation, you can set the debug mode by the `set_debug_mode` function.": "ステージのインスタンス後であれば`set_debug_mode`関数を使ってデバッグモードを設定ずることができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nap.set_debug_mode()\nsprite: ap.Sprite = ap.Sprite()\nint_1: ap.Int = ap.Int(10)\n\nap.save_overall_html(dest_dir_path="set_debug_mode_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nap.set_debug_mode()\nsprite: ap.Sprite = ap.Sprite()\nint_1: ap.Int = ap.Int(10)\n\nap.save_overall_html(dest_dir_path="set_debug_mode_basic_usage/")\n```',  # noqa
    ##################################################
    "This setting appends the information (Python's function and method callings, its module and class names, and argument information) as the JavaScript comment to the exported HTML, like the following:": "この設定は以下の例の用に出力されたHTML内にPythonの関数やメソッドの呼び出しやそのモジュールやクラス名、引数情報などのJavaScriptのコメントを追加します:",  # noqa
    ##################################################
    "```js\n...\n  //////////////////////////////////////////////////////////////////////\n  // [__init__ 1] started.\n  // module name: apysc._display.sprite\n  // class: Sprite\n  // arguments and variables:\n  //    variable_name = None\n  //    self = Sprite('')()\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 2] started.\n    // module name: apysc._type.array\n    // class: Array\n    // arguments and variables:\n    //    value = []\n    //    self = []()\n      //////////////////////////////////////////////////////////////////////\n      // [_append_constructor_expression 2] started.\n      // module name: apysc._type.array\n      // class: Array\n      // arguments and variables:\n      //    self = [](arr_2)\n        var arr_2 = [];\n      // [_append_constructor_expression 2] ended.\n      // module name: apysc._type.array\n      // class: Array\n      //////////////////////////////////////////////////////////////////////\n    // [__init__ 2] ended.\n    // module name: apysc._type.array\n    // class: Array\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 1] started.\n    // module name: apysc._display.display_object\n    // class: DisplayObject\n    // arguments and variables:\n    //    variable_name = 'sp_1'\n    //    self = Sprite('')()\n    // [__init__ 1] ended.\n    // module name: apysc._display.display_object\n    // class: DisplayObject\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [_append_constructor_expression 1] started.\n    // module name: apysc._display.sprite\n    // class: Sprite\n    // arguments and variables:\n    //    self = Sprite('sp_1')(sp_1)\n      var sp_1 = stage.nested();\n    // [_append_constructor_expression 1] ended.\n    // module name: apysc._display.sprite\n    // class: Sprite\n    //////////////////////////////////////////////////////////////////////\n...\n```": "```js\n...\n  //////////////////////////////////////////////////////////////////////\n  // [__init__ 1] started.\n  // module name: apysc._display.sprite\n  // class: Sprite\n  // arguments and variables:\n  //    variable_name = None\n  //    self = Sprite('')()\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 2] started.\n    // module name: apysc._type.array\n    // class: Array\n    // arguments and variables:\n    //    value = []\n    //    self = []()\n      //////////////////////////////////////////////////////////////////////\n      // [_append_constructor_expression 2] started.\n      // module name: apysc._type.array\n      // class: Array\n      // arguments and variables:\n      //    self = [](arr_2)\n        var arr_2 = [];\n      // [_append_constructor_expression 2] ended.\n      // module name: apysc._type.array\n      // class: Array\n      //////////////////////////////////////////////////////////////////////\n    // [__init__ 2] ended.\n    // module name: apysc._type.array\n    // class: Array\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 1] started.\n    // module name: apysc._display.display_object\n    // class: DisplayObject\n    // arguments and variables:\n    //    variable_name = 'sp_1'\n    //    self = Sprite('')()\n    // [__init__ 1] ended.\n    // module name: apysc._display.display_object\n    // class: DisplayObject\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [_append_constructor_expression 1] started.\n    // module name: apysc._display.sprite\n    // class: Sprite\n    // arguments and variables:\n    //    self = Sprite('sp_1')(sp_1)\n      var sp_1 = stage.nested();\n    // [_append_constructor_expression 1] ended.\n    // module name: apysc._display.sprite\n    // class: Sprite\n    //////////////////////////////////////////////////////////////////////\n...\n```",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [unset_debug_mode interface](unset_debug_mode.md)": "- [unset_debug_mode インターフェイス](jp_unset_debug_mode.md)",  # noqa
    ##################################################
    "## set_debug_mode API": "## set_debug_mode API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set the debug mode for the HTML and JavaScript debugging. If calling this function, this interface applies the following setting: ": "HTMLとJavaScriptのデバッグ用にデバッグモードの設定を行います。もしこの関数を呼び出した場合、のインターフェイスは以下の設定を追加します: ",  # noqa
    ##################################################
    "<br> ・Disabling HTML minify setting. ": "<br> ・HTMLの最小化（minify）設定を無効化します。 ",
    ##################################################
    "<br> ・Changing to append per each interface JavaScript divider string.<hr>": "<br> ・各インターフェイスごとのJavaScript上での区切りのための文字列を追加します。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> ap.set_debug_mode()\n>>> int_val: ap.Int = ap.Int(10)\n```": "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> ap.set_debug_mode()\n>>> int_val: ap.Int = ap.Int(10)\n```",  # noqa
}



"""The class implementation for the `content_cut` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContentCutIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `content_cut` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><circle cx="6" cy="18" fill="none" r="2"/><circle cx="12" cy="12" fill="none" r=".5"/><circle cx="6" cy="6" fill="none" r="2"/><path d="M9.64 7.64c.23-.5.36-1.05.36-1.64 0-2.21-1.79-4-4-4S2 3.79 2 6s1.79 4 4 4c.59 0 1.14-.13 1.64-.36L10 12l-2.36 2.36C7.14 14.13 6.59 14 6 14c-2.21 0-4 1.79-4 4s1.79 4 4 4 4-1.79 4-4c0-.59-.13-1.14-.36-1.64L12 14l7 7h3v-1L9.64 7.64zM6 8c-1.1 0-2-.89-2-2s.9-2 2-2 2 .89 2 2-.9 2-2 2zm0 12c-1.1 0-2-.89-2-2s.9-2 2-2 2 .89 2 2-.9 2-2 2zm6-7.5c-.28 0-.5-.22-.5-.5s.22-.5.5-.5.5.22.5.5-.22.5-.5.5zM19 3l-6 6 2 2 7-7V3z"/></svg>'  # noqa



"""The class implementation for the `pan_tool` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPanToolIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `pan_tool` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><g><path d="M23,5.5V20c0,2.2-1.8,4-4,4h-7.3c-1.08,0-2.1-0.43-2.85-1.19L1,14.83c0,0,1.26-1.23,1.3-1.25 c0.22-0.19,0.49-0.29,0.79-0.29c0.22,0,0.42,0.06,0.6,0.16C3.73,13.46,8,15.91,8,15.91V4c0-0.83,0.67-1.5,1.5-1.5S11,3.17,11,4v7 h1V1.5C12,0.67,12.67,0,13.5,0S15,0.67,15,1.5V11h1V2.5C16,1.67,16.67,1,17.5,1S19,1.67,19,2.5V11h1V5.5C20,4.67,20.67,4,21.5,4 S23,4.67,23,5.5z"/></g></g></g></svg>'  # noqa



"""The mix-in class implementation for the True and False constants classes.
"""

from typing import Union

from typing_extensions import Literal
from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int


class BoolConstMixIn:
    _value: bool

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_value_attr_with_value_arg(self, *, value: bool) -> None:
        """
        Set a `_value` attribute with a specified value argument.

        Notes
        -----
        This method's implementation does nothing (overridden as a blank method).

        Parameters
        ----------
        value : bool
            A boolean value.
        """

    @final
    def _set_value_and_skip_expression_appending(
        self, *, value: Union[bool, Literal[0, 1], Int, "Boolean"]
    ) -> None:
        """
        Update value attribute and skip expression appending.

        Notes
        -----
        This method's implementation does nothing (overridden as a blank method).

        Parameters
        ----------
        value : Boolean or Int or bool or int
            Any boolean value to set.
        """

    @final
    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Notes
        -----
        This method's implementation does nothing (overridden as a blank method).

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """

    @final
    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Notes
        -----
        This method's implementation does nothing (overridden as a blank method).

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """

    @property
    @add_debug_info_setting(module_name=__name__)
    def value(self) -> Union[bool, Literal[0, 1], Int, "Boolean"]:
        """
        Get a current boolean value.

        Returns
        -------
        value : bool
            Current boolean value.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> bool_val: ap.Boolean = ap.Boolean(True)
        >>> bool_val.value = False
        >>> bool_val.value
        False

        >>> bool_val.value = ap.Boolean(True)
        >>> bool_val.value
        True
        """
        return self._value

    @value.setter
    def value(self, value: Union[bool, Literal[0, 1], Int, "Boolean"]) -> None:
        """
        Set boolean value.

        Notes
        -----
        This interface is disabled and raise an exception if called.

        Parameters
        ----------
        value : Union[bool, Literal[0, 1], Int, &quot;Boolean&quot;]
            Any boolean value to set.

        Raises
        ------
        TypeError
            If this interface is called.
        """
        raise TypeError(
            "The `True` or `False` value is constant and cannot be changed."
        )



"""The class implementation for the `fiber_dvr` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFiberDvrIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `fiber_dvr` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><g><path d="M17.5,10.5h2v1h-2V10.5z M4.5,10.5h2v3h-2V10.5z M21,3H3C1.89,3,1,3.89,1,5v14c0,1.1,0.89,2,2,2h18c1.11,0,2-0.9,2-2V5 C23,3.89,22.11,3,21,3z M8,13.5C8,14.35,7.35,15,6.5,15H3V9h3.5C7.35,9,8,9.65,8,10.5V13.5z M12.62,15h-1.5L9.37,9h1.5l1,3.43 l1-3.43h1.5L12.62,15z M21,11.5c0,0.6-0.4,1.15-0.9,1.4L21,15h-1.5l-0.85-2H17.5v2H16V9h3.5c0.85,0,1.5,0.65,1.5,1.5V11.5z"/></g></g></g></svg>'  # noqa



"""The mix-in class for the `_add_foreign_object_child` method.
"""

from typing_extensions import final

from apysc._display.svg_foreign_object_child import SvgForeignObjectChild
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class AddForeignObjectChildMixIn:
    @final
    @arg_validation_decos.is_svg_foreign_object_child(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def _add_foreign_object_child(self, *, child: SvgForeignObjectChild) -> None:
        """
        Add a foreignObject's child object.

        Parameters
        ----------
        child : SvgForeignObjectChild
            A foreignObject's child object.
        """
        from apysc._expression import expression_data_util
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        self_variable_name: str = validate_variable_name_mixin_type(
            instance=self
        ).variable_name
        expression: str = f"{self_variable_name}.add({child.variable_name});"
        expression_data_util.append_js_expression(expression=expression)



"""This module is for the attribute or variable name's suffix-related
mix-in class.
"""

from typing_extensions import final

from apysc._validation import arg_validation_decos


class VariableNameSuffixAttrOrVarMixIn:
    @final
    @arg_validation_decos.not_empty_string(arg_position_index=1)
    def _get_attr_or_variable_name_suffix(self, *, value_identifier: str) -> str:
        """
        Get an attribute or variable name's suffix if its value
        is not blank.

        Parameters
        ----------
        value_identifier : str
            Value identifier string (e.g., `x`).

        Returns
        -------
        attr_or_variable_name_suffix : str
            An attribute or variable's name suffix.
            In the following cases, this value becomes a blank string.
            - If this instance is not the `VariableNameSuffixMixIn`
                instance.
            - If a suffix is a blank string.
        """
        from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn

        if not isinstance(self, VariableNameSuffixMixIn):
            return ""
        if self._variable_name_suffix == "":
            return ""

        attr_or_variable_name_suffix: str = (
            f"{self._variable_name_suffix}__{value_identifier}"
        )
        return attr_or_variable_name_suffix



"""The mix-in class for the `_append_foreign_object_constructor_expression`
method.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class AppendForeignObjectConstructorExpressionMixIn:
    _width: Int

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_foreign_object_constructor_expression(self) -> None:
        """
        Append a `foreignObject`'s constructor expression.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        self_variable_name: str = validate_variable_name_mixin_type(
            instance=self
        ).variable_name
        stage: Stage = get_stage()
        expression: str = (
            f"var {self_variable_name} = {stage.variable_name}"
            ".foreignObject("
            f"{self._width.variable_name}, 0);"
        )
        expression_data_util.append_js_expression(expression=expression)



"""The interface class implementation for the `_get_last_scope` method.
"""

from abc import ABC
from abc import abstractmethod

from apysc._expression.last_scope import LastScope


class GetLastScopeInterface(ABC):
    @abstractmethod
    def _get_last_scope(self) -> LastScope:
        """
        Get a target last scope value.
        """



"""This module is for the translation mapping data of the
following document:

Document file: array_sort.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class sort interface": "# Array クラスの sort インターフェイス",
    ##################################################
    "This page explains the `Array` class `sort` method interface.": "このページでは`Array`クラスの`sort`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `sort` method interface sorts an array's values (default is ascending order).": "`sort`メソッドのインターフェイスは配列の値をソートします（デフォルトでは昇順となります）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `sort` method requires no arguments and sorts values ascending in place (no return value).": "`sort`は昇順でソートする場合引数を必要とせず、配列の値は直接更新されます（返却値は設定されません）。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([5, 1, 3])\narr.sort()\nassert arr == [1, 3, 5]\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([5, 1, 3])\narr.sort()\nassert arr == [1, 3, 5]\n```",  # noqa
    ##################################################
    "If you specify `False` to the `ascending` option, a result value becomes descending order.": "もし`ascending`引数に`False`を指定した場合、結果は降順となります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([5, 1, 3])\narr.sort(ascending=False)\nassert arr == [5, 3, 1]\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([5, 1, 3])\narr.sort(ascending=False)\nassert arr == [5, 3, 1]\n```",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Array class reverse interface](array_reverse.md)": "- [Array クラスの reverse インターフェイス](jp_array_reverse.md)",  # noqa
    ##################################################
    "## sort API": "## sort API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Sort this array in place.<hr>": "この配列の値を直接ソートします。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `ascending`: bool, default True": "- `ascending`: bool, default True",
    ##################################################
    "  - Sort by ascending or not. If False is specified, this interface sorts values descending.": "  - 昇順でソートを行うかどうかの指定です。もしFalseが指定された場合、このインターフェイスは降順で値をソートします。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([3, 5, 1, 4, 2])\n>>> arr.sort()\n>>> arr\nArray([1, 2, 3, 4, 5])\n\n>>> arr.sort(ascending=False)\n>>> arr\nArray([5, 4, 3, 2, 1])\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([3, 5, 1, 4, 2])\n>>> arr.sort()\n>>> arr\nArray([1, 2, 3, 4, 5])\n\n>>> arr.sort(ascending=False)\n>>> arr\nArray([5, 4, 3, 2, 1])\n```",  # noqa
}



"""This module is for the `DisplayObject` subclass implementation
to avoid an abstract method error.
"""

from typing_extensions import final

from apysc._display.css_mixin import CssMixIn
from apysc._display.display_object import DisplayObject
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.use_hand_cursor_mixin import UseHandCursorMixIn
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._expression import var_names
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)


class AnyDisplayObject(
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    DisplayObject,
    XMixIn,
    YMixIn,
    UseHandCursorMixIn,
    InitializeWithBaseValueInterface,
):
    """
    This class is for the `DisplayObject` subclass
    implementation to avoid an abstract method error.
    """

    @add_debug_info_setting(module_name=__name__)
    def __init__(self) -> None:
        """
        This class is for the `DisplayObject` subclass
        implementation to avoid an abstract method error.
        """
        from apysc._expression import expression_data_util
        from apysc._expression.expression_variables_util import get_next_variable_name

        variable_name: str = get_next_variable_name(
            type_name=var_names.ANY_DISPLAY_OBJECT
        )
        super(AnyDisplayObject, self).__init__(variable_name=variable_name)
        expression: str = f"var {variable_name};"
        expression_data_util.append_js_expression(expression=expression)

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "AnyDisplayObject":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        any_display_object : AnyDisplayObject
            An initialized object.
        """
        from apysc._type.boolean import Boolean

        any_display_object: AnyDisplayObject = AnyDisplayObject()
        any_display_object.visible = Boolean(False)
        return any_display_object



"""The class implementation for the `thumb_up_off_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialThumbUpOffAltIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `thumb_up_off_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M13.11 5.72l-.57 2.89c-.12.59.04 1.2.42 1.66.38.46.94.73 1.54.73H20v1.08L17.43 18H9.34c-.18 0-.34-.16-.34-.34V9.82l4.11-4.1M14 2L7.59 8.41C7.21 8.79 7 9.3 7 9.83v7.83C7 18.95 8.05 20 9.34 20h8.1c.71 0 1.36-.37 1.72-.97l2.67-6.15c.11-.25.17-.52.17-.8V11c0-1.1-.9-2-2-2h-5.5l.92-4.65c.05-.22.02-.46-.08-.66-.23-.45-.52-.86-.88-1.22L14 2zM4 9H2v11h2c.55 0 1-.45 1-1v-9c0-.55-.45-1-1-1z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_polygon.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_polygon interface": "# Graphics クラスの draw_polygon インターフェイス",
    ##################################################
    "This page explains the `Graphics` class `draw_polygon` method interface.": "このページでは`Graphics`クラスの`draw_polygon`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `draw_polygon` interface draws vector polygon graphics. This interface works slightly similar to the `line_to` and `move_to` interfaces, but the paths do not need to be closed.": "`draw_polygon`インターフェイスは多角形のベクターグラフィックスを描画します。このインターフェイスは`line_to`や`move_to`などのインターフェイスと挙動が少し似ていますが、パスを閉じなくても良いという違いがあります。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `draw_polygon` interface has the `points` argument, which determines the polygon vertices coordinates.": "`draw_polygon`インターフェイスは各頂点の座標を決めるための`points`引数を必要とします。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=100),\n            ap.Point2D(x=100, y=100),\n        ]\n    )\n)\n\n# Draw the diamond shape with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=175, y=50),\n            ap.Point2D(x=150, y=75),\n            ap.Point2D(x=175, y=100),\n            ap.Point2D(x=200, y=75),\n        ]\n    )\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=100),\n            ap.Point2D(x=100, y=100),\n        ]\n    )\n)\n\n# Draw the diamond shape with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=175, y=50),\n            ap.Point2D(x=150, y=75),\n            ap.Point2D(x=175, y=100),\n            ap.Point2D(x=200, y=75),\n        ]\n    )\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_basic_usage/")\n```',  # noqa
    ##################################################
    "## Difference between the line_to and draw_polygon interfaces": "## line_to と draw_polygon の各インターフェイスの違いについて",  # noqa
    ##################################################
    "If you set the fill color, the `draw_polygon` interface becomes slightly similar to the `line_to` (and `move_to`) interfaces. So, for example, the following codes both draw the triangle.": "塗りの色の設定をした場合`draw_polygon`と`line_to`（及び`move_to`）のインターフェイスの挙動は少し近くなります。例えば以下のコード例では各インターフェイスでどちらも三角形が描画しています。",  # noqa
    ##################################################
    "The `draw_polygon` interface draws the left triangle. Similarly, the `move_to` and `line_to` interfaces draw the right one.": "`draw_polygon`インターフェイスでは左側の三角形を描画し、`move_to`と`line_to`のインターフェイスでは右側の三角形を描画しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=100),\n            ap.Point2D(x=100, y=100),\n        ]\n    )\n)\n\n# Draw the triangle with the move_to and line_to interfaces.\nsprite.graphics.move_to(x=175, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_line_to_difference_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=100),\n            ap.Point2D(x=100, y=100),\n        ]\n    )\n)\n\n# Draw the triangle with the move_to and line_to interfaces.\nsprite.graphics.move_to(x=175, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_line_to_difference_1/")\n```',  # noqa
    ##################################################
    "But there is a difference in whether closing the paths is necessary or not. This difference becomes significant when you set the line style setting. The `line_to` interface does not close the paths from end coordinates to start coordinates.": "一方で、各インターフェイスにはパスを閉じる必要があるかどうかの違いがあります。この違いは線の設定を行った場合には顕著になります。`line_to`のインターフェイスでは終点の座標から始点の座標へはパスが自動では繋がりません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Set the line style to see the difference.\nsprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=100),\n            ap.Point2D(x=100, y=100),\n        ]\n    )\n)\n\n# Draw the triangle with the move_to and line_to interfaces.\nsprite.graphics.move_to(x=175, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_line_to_difference_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Set the line style to see the difference.\nsprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=100),\n            ap.Point2D(x=100, y=100),\n        ]\n    )\n)\n\n# Draw the triangle with the move_to and line_to interfaces.\nsprite.graphics.move_to(x=175, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_line_to_difference_2/")\n```',  # noqa
    ##################################################
    "## Return value": "## 返却値",
    ##################################################
    "The `draw_polygon` interface returns the `Polygon` instance. And that has the basic interface as same as the other type graphics instances. The `Polygon` instance also has the `append_line_point` method interface to append points dynamically.": "`draw_polygon`インターフェイスは`Polygon`クラスのインスタンスを返却します。そのインスタンスは他のグラフィックス系のインスタンスと同様の基本的なインターフェイスを持っています。加えて、`Polygon`クラスは頂点を加えるための`append_line_point`メソッドを持っています。",  # noqa
    ##################################################
    "For instance, the following code appends the point and changes from the triangle to the rectangle.": "例えば以下のコード例では座標の追加を行い三角から四角に変換しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Draw the triangle.\npolygon: ap.Polygon = sprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=75),\n            ap.Point2D(x=75, y=100),\n        ]\n    )\n)\n\n# Append the point and change to the rectangle dynamically.\npolygon.append_line_point(x=100, y=75)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_append_line_point/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Draw the triangle.\npolygon: ap.Polygon = sprite.graphics.draw_polygon(\n    points=ap.Array(\n        [\n            ap.Point2D(x=75, y=50),\n            ap.Point2D(x=50, y=75),\n            ap.Point2D(x=75, y=100),\n        ]\n    )\n)\n\n# Append the point and change to the rectangle dynamically.\npolygon.append_line_point(x=100, y=75)\n\nap.save_overall_html(dest_dir_path="graphics_draw_polygon_append_line_point/")\n```',  # noqa
    ##################################################
    "## draw_polygon API": "## draw_polygon API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a polygon vector graphic. This interface is similar to the Polyline class (created by `move_to` or `line_to`). But unlike that, this interface connects the last point and the start point.<hr>": "多角形のベクターグラフィックスを描画します。このインターフェイスはPolylineクラス（`move_to`や`line_to`のインターフェイスで作成されます）に似ていますが、このインターフェイスは始点と終点が連結されるという違いがあります。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `points`: list of Point2D or Array.": "- `points`: list of Point2D or Array.",
    ##################################################
    "  - Polygon vertex points.": "  - 多角形の頂点の各座標。",
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `polygon`: Polygon": "- `polygon`: Polygon",
    ##################################################
    "  - Created polygon graphics instance.": "  - 作成された多角形のグラフィックスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=25, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=50),\n...     ]\n... )\n>>> polygon.fill_color\nColor("#00aaff")\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=25, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=50),\n...     ]\n... )\n>>> polygon.fill_color\nColor("#00aaff")\n```',  # noqa
}



"""The mix-in class implementation for the `rstrip` method.
"""

from typing import TYPE_CHECKING
from typing import Any
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._type.string import String


class StringRStripMixIn:
    @final
    @arg_validation_decos.is_apysc_string(arg_position_index=0)
    @arg_validation_decos.is_string(arg_position_index=1, optional=True)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def rstrip(
        self,
        *,
        string: Optional[Union[str, "String"]] = None,
        variable_name_suffix: str = "",
    ) -> "String":
        """
        Remove a specified character or string from the end of this value.

        Parameters
        ----------
        string : Optional[Union[str, "String"]], optional
            A character or string to remove from the end of this value.
            If this argument is `None` (default), this method removes
            spaces and line breaks.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        result : String
            A stripped result string.

        References
        ----------
        - String class rstrip interface
            - https://simon-ritchie.github.io/apysc/en/string_rstrip.html

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage()
        >>> string: ap.String = ap.String("  aabbcc   ")
        >>> string = string.rstrip()
        >>> string
        String("  aabbcc")

        >>> string = ap.String("aabbccaa")
        >>> string = string.rstrip(string="a")
        >>> string
        String("aabbcc")
        """
        from apysc._expression import expression_data_util
        from apysc._type.string import String
        from apysc._type.variable_name_mixin import VariableNameMixIn

        result: String = String("", variable_name_suffix=variable_name_suffix)
        self_variable_name: str = ""
        if isinstance(self, VariableNameMixIn):
            self_variable_name = self.variable_name
        if string is None:
            expression: str = _create_string_none_case_expression(
                result_string=result,
                self_variable_name=self_variable_name,
            )
        else:
            expression = _create_string_not_none_case_expression(
                result_string=result,
                removing_string=string,
                self_variable_name=self_variable_name,
                variable_name_suffix=variable_name_suffix,
            )
        expression_data_util.append_js_expression(expression=expression)
        py_str: str = _get_py_str_from_current_value(
            self_str=self,
            removing_string=string,
        )
        result._value = py_str
        return result


@arg_validation_decos.is_apysc_string(arg_position_index=0)
@arg_validation_decos.is_string(arg_position_index=1, optional=True)
def _get_py_str_from_current_value(
    *,
    self_str: Any,
    removing_string: Optional[Union[str, "String"]],
) -> str:
    """
    Get a Python string from a current string value.

    Parameters
    ----------
    self_str : Any
        A self-string.
    removing_string : Optional[Union[str, "String"]]
        A removing target string.

    Returns
    -------
    py_str : str
        A Python string.
    """
    from apysc._type.string import String

    py_str: str = ""
    if isinstance(self_str, String):
        py_str = self_str._value
    if removing_string is None:
        py_str = (
            py_str.rstrip()
            .rstrip("\\r\\n")
            .rstrip("\\n")
            .rstrip("\\r")
            .rstrip("\\t")
            .rstrip()
        )
        return py_str
    if isinstance(removing_string, String):
        py_str = py_str.rstrip(removing_string._value)
        return py_str
    if isinstance(removing_string, str):
        py_str = py_str.rstrip(removing_string)
    return py_str


@arg_validation_decos.is_apysc_string(arg_position_index=0)
@arg_validation_decos.is_string(arg_position_index=1, optional=False)
@arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
@arg_validation_decos.is_builtin_string(arg_position_index=3, optional=False)
def _create_string_not_none_case_expression(
    *,
    result_string: "String",
    removing_string: Union[str, "String"],
    self_variable_name: str,
    variable_name_suffix: str,
) -> str:
    """
    Create an expression for the string's not `None` case.

    Parameters
    ----------
    result_string : String
        A result string.
    removing_string : Union[str, "String"]
        A removing target string.
    self_variable_name : str
        An instance's self-variable name.
    variable_name_suffix : str
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    expression : str
        A created expression.
    """
    from apysc._type.string import String

    if not isinstance(removing_string, String):
        removing_string_: String = String(
            removing_string, variable_name_suffix=variable_name_suffix
        )
    else:
        removing_string_ = removing_string
    expression: str = (
        f"{result_string.variable_name} = {self_variable_name}"
        f'.replace(new RegExp(`(${{{removing_string_.variable_name}}})+$`), "");'
    )
    return expression


@arg_validation_decos.is_apysc_string(arg_position_index=0)
@arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
def _create_string_none_case_expression(
    *,
    result_string: "String",
    self_variable_name: str,
) -> str:
    """
    Create an expression for the string's `None` case.

    Parameters
    ----------
    result_string : String
        A result string.
    self_variable_name : str
        An instance's self-variable name.

    Returns
    -------
    expression : str
        A created expression.
    """
    expression: str = f"{result_string.variable_name} = {self_variable_name}.trimEnd();"
    return expression



"""The constant definition for the `Colorless` class.
"""

from typing_extensions import Final

from apysc._color.color import Color


class Colorless(Color):
    """
    The constant class for the colorlessness.
    """

    def __init__(self) -> None:
        """
        The constant class for the colorlessness.
        """
        super(Colorless, self).__init__(value="", variable_name_suffix="colorless")


COLORLESS: Final[Colorless] = Colorless()



"""The class implementation for the `backspace` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBackspaceIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `backspace` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M22 3H7c-.69 0-1.23.35-1.59.88L0 12l5.41 8.11c.36.53.9.89 1.59.89h15c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-3 12.59L17.59 17 14 13.41 10.41 17 9 15.59 12.59 12 9 8.41 10.41 7 14 10.59 17.59 7 19 8.41 15.41 12 19 15.59z"/></svg>'  # noqa



"""Class implementation for the polygon-related classes' y2 mix-in.
"""

from apysc._display.polygon_apply_current_points_mixin import (
    PolygonApplyCurrentPointsMixIn,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class PolygonY2MixIn(PolygonApplyCurrentPointsMixIn):
    _y2: Number

    @property
    @add_debug_info_setting(module_name=__name__)
    def y2(self) -> Number:
        """
        Get a second y-coordinate.

        Returns
        -------
        y2 : Number
            A second y-coordinate.
        """
        y2: Number = self._points[1].y
        return y2

    @y2.setter
    @add_debug_info_setting(module_name=__name__)
    def y2(self, value: Number) -> None:
        """
        Set a second y-coordinate.

        Parameters
        ----------
        value : Number
            Y-coordinate to set.
        """
        from apysc._geom.point2d import Point2D

        self._y2._value = value._value
        new_point: Point2D = self._points[1]
        new_point.y = value
        self._points[1] = new_point
        self._apply_current_points()



"""This module is for the translation mapping data of the
following document:

Document file: string_strip.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class strip interface": "# String クラスの strip インターフェイス",
    ##################################################
    "This page explains the `String` class `strip` method interface.": "このページでは`String`クラスの`strip`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `strip` method interface removes whitespaces or specified character(s) from a string's beginning and end edges.": "`strip`メソッドのインターフェイスは文字列の左端と右端から空白文字もしくは指定された文字（文字列）を取り除きます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `strip` method accepts the optional `string` argument.": "`strip`メソッドは省略可能な`string`引数を受け付けます。",  # noqa
    ##################################################
    "If you skip this argument, this interface removes whitespaces (and line breaks) from a string's beginning and end edges.": "もしもこの引数の指定を省略した場合、このインターフェイスは文字列の左端と右端からスペースや改行などを取り除きます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String("  \n  aabbaa  ")\nstring = string.strip()\nap.assert_equal(string, "aabbaa")\n\nap.save_overall_html(dest_dir_path="string_strip_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String("  \n  aabbaa  ")\nstring = string.strip()\nap.assert_equal(string, "aabbaa")\n\nap.save_overall_html(dest_dir_path="string_strip_basic_usage_1/")\n```',  # noqa
    ##################################################
    "Also, if you specify any value to the `string` argument, this interface removes its character(s) from a string's beginning and end edges.": "また、もし`string`引数に何らかの値を指定した場合、このインターフェイスは文字列の左端と右端から指定された文字（もしくは文字列）を取り除きます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String("aabbccaa")\nstring = string.strip(string="a")\nap.assert_equal(string, "bbcc")\n\nap.save_overall_html(dest_dir_path="string_strip_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstring: ap.String = ap.String("aabbccaa")\nstring = string.strip(string="a")\nap.assert_equal(string, "bbcc")\n\nap.save_overall_html(dest_dir_path="string_strip_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## strip API": "## strip API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Remove a specified character or string from left- and right-edges.<hr>": "指定された文字もしくは文字列を左端と右端から取り除きます。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    '- `string`: Optional[Union[str, "String"]], optional': '- `string`: Optional[Union[str, "String"]], optional',  # noqa
    ##################################################
    "  - A character or string to remove from the beginning and end of the this value. If this argument is `None` (default), this method removes spaces and line breaks.": "  - この値の左端と右端から取り除く文字もしくは文字列。もし`None`（デフォルト値）が指定された場合、このメソッドはスペースや改行などを取り除きます。",  # noqa
    ##################################################
    "- `variable_name_suffix`: str, optional": "- `variable_name_suffix`: str, optional",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result`: String": "- `result`: String",
    ##################################################
    "  - A stripped result string.": "  - 除外処理実行後の文字列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> string: ap.String = ap.String("   aabbcc   ")\n>>> string = string.strip()\n>>> string\nString("aabbcc")\n\n>>> string = ap.String("aabbccaa")\n>>> string = string.strip(string="a")\n>>> string\nString("bbcc")\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> string: ap.String = ap.String("   aabbcc   ")\n>>> string = string.strip()\n>>> string\nString("aabbcc")\n\n>>> string = ap.String("aabbccaa")\n>>> string = string.strip(string="a")\n>>> string\nString("bbcc")\n```',  # noqa
}



from abc import ABC
from abc import abstractmethod
from typing import Union

from apysc._type.string import String


class CssInterface(ABC):
    @abstractmethod
    def get_css(self, *, name: Union[str, String]) -> String:
        """
        Get a CSS value string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').

        Returns
        -------
        css : ap.String
            CSS value.

        References
        ----------
        - Display object get_css and set_css interfaces
            - https://simon-ritchie.github.io/apysc/en/display_object_get_and_set_css.html  # noqa
        """
        return String("")

    @abstractmethod
    def set_css(self, *, name: Union[str, String], value: Union[str, String]) -> None:
        """
        Set a specified value string to the CSS.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        value : str or String
            A CSS value string (e.g., 'none').

        References
        ----------
        - Display object get_css and set_css interfaces
            - https://simon-ritchie.github.io/apysc/en/display_object_get_and_set_css.html  # noqa
        """



"""Mix-in class implementation for the destination y path data.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class PathDestYMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
    VariableNameSuffixMixIn,
):
    _dest_y: Number

    @final
    def _initialize_dest_y_if_not_initialized(self) -> None:
        """
        Initialize the _dest_y attribute if this instance
        does not initialize it yet.
        """
        if hasattr(self, "_dest_y"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="dest_y")
        self._dest_y = Number(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_dest_y_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_dest_y_linking_setting(self) -> None:
        """
        Append a dest_y linking setting.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._dest_y, attr_name="dest_y"
        )
        self._append_attr_to_linking_stack(attr=self._dest_y, attr_name="dest_y")

    @property
    @add_debug_info_setting(module_name=__name__)
    def dest_y(self) -> Number:
        """
        Get a y-coordinate of the destination point.

        Returns
        -------
        dest_y : Number
            Y-coordinate of the destination point.

        Examples
        --------
        >>> import apysc as ap
        >>> bezier_2d: ap.PathBezier2D = ap.PathBezier2D(
        ...     control_x=50, control_y=0, dest_x=100, dest_y=50
        ... )
        >>> bezier_2d.dest_y = ap.Number(75)
        >>> bezier_2d.dest_y
        Number(75.0)
        """
        self._initialize_dest_y_if_not_initialized()
        return self._dest_y.copy()

    @dest_y.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def dest_y(self, value: Number) -> None:
        """
        Set a y-coordinate of the destination point.

        Parameters
        ----------
        value : Number
            Y-coordinate of the destination point
        """
        self._initialize_dest_y_if_not_initialized()
        self._dest_y.value = value

        self._append_dest_y_linking_setting()

    _dest_y_snapshots: Optional[Dict[str, float]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_dest_y_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_dest_y_snapshots",
            value=float(self._dest_y._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_dest_y_if_not_initialized()
        self._dest_y._value = self._get_snapshot_val_if_exists(
            current_value=self._dest_y._value,
            snapshot_dict=self._dest_y_snapshots,
            snapshot_name=snapshot_name,
        )



"""This module is for the translation mapping data of the
following document:

Document file: animation_parallel.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_parallel interface": "# animation_parallel インターフェイス",
    ##################################################
    "This page explains the `animation_parallel` interface.": "このページでは`animation_parallel`のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `animation_parallel` method interface creates an `AnimationParallel` instance. You can set multiple simultaneous animations to the same instance with it.": "`animation_parallel`メソッドのインターフェイスは`AnimationParallel`クラスのインスタンスを生成します。このインスタンスを使うことで複数のアニメーションの同時実行の設定を行うことができます。",  # noqa
    ##################################################
    "This interface exists on a `DisplayObject` subclass instance, such as the `Sprite` or `Rectangle` class.": "このインターフェイスは`Sprite`や`Rectangle`などの`DisplayObject`の各サブクラスに存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can use this interface with the `animation_parallel` method. The `animations` list argument does not require any animation settings.": "`animation_parallel`メソッドを使ってこのインターフェイスを使用することができます。`animations`引数の値は各アニメーションの設定値の指定を必要としません。",  # noqa
    ##################################################
    "The following example sets the simultaneous animations of the x, fill alpha, fill color, and line thickness.": "以下のコード例ではX座標、塗りの透明度、塗りの色、そして線の幅に対するアニメーションを同時に実行しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=400,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_parallel(\n    animations=[\n        rectangle.animation_x(x=300),\n        rectangle.animation_fill_color(fill_color=ap.Color("#f0a")),\n        rectangle.animation_fill_alpha(alpha=0.3),\n        rectangle.animation_line_thickness(thickness=7),\n    ],\n    duration=3000,\n    delay=3000,\n    easing=ap.Easing.EASE_OUT_QUINT,\n).start()\n\nap.save_overall_html(dest_dir_path="animation_parallel_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=400,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_parallel(\n    animations=[\n        rectangle.animation_x(x=300),\n        rectangle.animation_fill_color(fill_color=ap.Color("#f0a")),\n        rectangle.animation_fill_alpha(alpha=0.3),\n        rectangle.animation_line_thickness(thickness=7),\n    ],\n    duration=3000,\n    delay=3000,\n    easing=ap.Easing.EASE_OUT_QUINT,\n).start()\n\nap.save_overall_html(dest_dir_path="animation_parallel_basic_usage/")\n```',  # noqa
    ##################################################
    "## Note for each animation's duration, delay, and easing setting": "## 各アニメーションのduration、delay、easingの設定についての特記事項",  # noqa
    ##################################################
    "Animation settings of the `duration`\\, `delay`\\, and `easing` in the `animations` argument can't be changed since these animation settings are referring to the `animation_parallel` arguments. If you set these parameters in the `animations` arguments, setting raise an error:": "各アニメーションごとの`duration`や`delay`、`easing`の引数設定は`animation_parallel`メソッド側の値が使用されるため設定できません。もし何らかの値を設定した場合にはエラーとなります。",  # noqa
    ##################################################
    "```py\n...\nrectangle.animation_parallel(\n    animations=[\n        rectangle.animation_x(x=300, duration=1000),\n    ],\n    duration=3000,\n    delay=2000,\n    easing=ap.Easing.EASE_OUT_QUINT,\n)\n...\n```": "```py\n...\nrectangle.animation_parallel(\n    animations=[\n        rectangle.animation_x(x=300, duration=1000),\n    ],\n    duration=3000,\n    delay=2000,\n    easing=ap.Easing.EASE_OUT_QUINT,\n)\n...\n```",  # noqa
    ##################################################
    "```\nValueError: There is an animation target that is changed duration setting: 1000\nThe duration setting of animation in the `animations` argument can not be changed.\nTarget animation type: <class 'apysc._animation.animation_x.AnimationX'>\n```": "```\nValueError: There is an animation target that is changed duration setting: 1000\nThe duration setting of animation in the `animations` argument can not be changed.\nTarget animation type: <class 'apysc._animation.animation_x.AnimationX'>\n```",  # noqa
    ##################################################
    "## animation_parallel API": "## animation_parallel API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set the parallel animation setting.<hr>": "並列実行されるアニメーション設定を行います。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `animations`: list of AnimationBase": "- `animations`: list of AnimationBase",
    ##################################################
    "  - Target animation settings.": "  - 対象の各アニメーションの設定。",
    ##################################################
    "- `duration`: Int or int, default 3000": "- `duration`: Int or int, default 3000",
    ##################################################
    "  - Milliseconds before an animation ends.": "  - アニメーション完了までのミリ秒。",
    ##################################################
    "- `delay`: Int or int, default 0": "- `delay`: Int or int, default 0",
    ##################################################
    "  - Milliseconds before an animation starts.": "  - アニメーション開始までの遅延時間のミリ秒。",
    ##################################################
    "- `easing`: Easing, default Easing.LINEAR": "- `easing`: Easing, default Easing.LINEAR",  # noqa
    ##################################################
    "  - Easing setting.": "  - イージング設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `animation_parallel`: AnimationParallel": "- `animation_parallel`: AnimationParallel",  # noqa
    ##################################################
    "  - Created animation setting instance.": "  - 生成されたアニメーションのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Raises]**": "**[エラー発生条件]**",
    ##################################################
    "- ValueError: ": "- ValueError: ",
    ##################################################
    "<br> ・If the animations' target is not this instance. ": "<br> ・もし各アニメーションの対象のインスタンスがこのインスタンスでは無い場合。",  # noqa
    ##################################################
    "<br> ・If there are changed duration, delay, or easing animation settings in the `animations` list.": "<br> ・もし`animations`引数の各アニメーションのderationやdelay、easingの設定が変更去れている場合。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    " ・To start this animation, you need to call the `start` method of the returned instance. ": " ・アニメーションを開始するには返却されたインスタンスの`start`メソッドを呼び出す必要があります。 ",  # noqa
    ##################################################
    "<br> ・The `animations` argument can't contains the `AnimationParallel` instance. ": "<br> ・`animations`引数の値には`AnimationParallel`クラスのインスタンスを含めることはできません。 ",  # noqa
    ##################################################
    "<br> ・This interface ignores the duration, delay, and easing arguments in the `animations` argument (this interface uses self-arguments instead).<hr>": "<br> ・このインターフェイスは`animations`引数内の値のduration, delay, easingの各引数の設定を無視します（代わりにこのインターフェイス自身のそれらの引数の設定を利用してください）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_parallel(\n...     animations=[\n...         rectangle.animation_x(x=100),\n...         rectangle.animation_fill_color(fill_color=ap.Color("#f0a")),\n...         rectangle.animation_fill_alpha(alpha=0.5),\n...     ],\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_parallel(\n...     animations=[\n...         rectangle.animation_x(x=100),\n...         rectangle.animation_fill_color(fill_color=ap.Color("#f0a")),\n...         rectangle.animation_fill_alpha(alpha=0.5),\n...     ],\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Animation interfaces duration setting](https://simon-ritchie.github.io/apysc/en/animation_duration.html)": "- [各アニメーションインターフェイスの duration （アニメーション時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_duration.html)",  # noqa
    ##################################################
    "- [Animation interfaces delay setting](https://simon-ritchie.github.io/apysc/en/animation_delay.html)": "- [各アニメーションインターフェイスの delay （遅延時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_delay.html)",  # noqa
    ##################################################
    "- [Each animation interface return value](https://simon-ritchie.github.io/apysc/en/animation_return_value.html)": "- [各アニメーションインターフェイスの返却値](https://simon-ritchie.github.io/apysc/jp/jp_animation_return_value.html)",  # noqa
    ##################################################
    "- [Sequential animation setting](https://simon-ritchie.github.io/apysc/en/sequential_animation.html)": "- [連続したアニメーション設定](https://simon-ritchie.github.io/apysc/jp/jp_sequential_animation.html)",  # noqa
    ##################################################
    "- [Easing enum](https://simon-ritchie.github.io/apysc/en/easing_enum.html)": "- [イージングのenum](https://simon-ritchie.github.io/apysc/jp/jp_easing_enum.html)",  # noqa
}



"""The class implementation for the outlined `settings_input_svideo` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSettingsInputSvideoOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `settings_input_svideo` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M8 11.5c0-.83-.67-1.5-1.5-1.5S5 10.67 5 11.5 5.67 13 6.5 13 8 12.33 8 11.5zm7-5c0-.83-.67-1.5-1.5-1.5h-3C9.67 5 9 5.67 9 6.5S9.67 8 10.5 8h3c.83 0 1.5-.67 1.5-1.5zM8.5 15c-.83 0-1.5.67-1.5 1.5S7.67 18 8.5 18s1.5-.67 1.5-1.5S9.33 15 8.5 15zM12 1C5.93 1 1 5.93 1 12s4.93 11 11 11 11-4.93 11-11S18.07 1 12 1zm0 20c-4.96 0-9-4.04-9-9s4.04-9 9-9 9 4.04 9 9-4.04 9-9 9zm5.5-11c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm-2 5c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: array_pop.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class pop interface": "# Array クラスの pop インターフェイス",
    ##################################################
    "This page explains the `Array` class `pop` method interface.": "このページでは`Array`クラスの`pop`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `pop` method interface removes the last value from an array and returns that value.": "`pop`メソッドは配列内の最後の値を配列から取り除き、そしてその取り除いた値を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `pop` method accepts no arguments and returns the last value, as follows:": "`pop`メソッドには必要な引数は無く、以下のコード例のように配列の最後の値を返却します:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 2, 3])\nlast_value: int = arr.pop()\nassert last_value == 3\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 2, 3])\nlast_value: int = arr.pop()\nassert last_value == 3\n```",  # noqa
    ##################################################
    "## pop API": "## pop API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Remove this array's last value and return it.<hr>": "この配列の末尾の値を取り除き、その値を返却します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Removed value.": "  - 取り除かれた値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> popped_val: int = arr.pop()\n>>> popped_val\n3\n\n>>> arr\nArray([1, 2])\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> popped_val: int = arr.pop()\n>>> popped_val\n3\n\n>>> arr\nArray([1, 2])\n```",  # noqa
}



"""The class implementation for the outlined `opacity` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialOpacityOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `opacity` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M24 0H0v24h24V0zm0 0H0v24h24V0zM0 24h24V0H0v24z" fill="none"/><path d="M17.66 8L12 2.35 6.34 8C4.78 9.56 4 11.64 4 13.64s.78 4.11 2.34 5.67 3.61 2.35 5.66 2.35 4.1-.79 5.66-2.35S20 15.64 20 13.64 19.22 9.56 17.66 8zM6 14c.01-2 .62-3.27 1.76-4.4L12 5.27l4.24 4.38C17.38 10.77 17.99 12 18 14H6z"/></svg>'  # noqa



"""The mix-in class implementation for the CSS line-height property.
"""

from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class TextLineHeightCssMixIn(
    AttrLinkingMixIn,
):
    _line_height: Number

    def _initialize_line_height(self) -> None:
        """
        Initialize the `_line_height` attribute.
        """
        if hasattr(self, "_line_height"):
            return
        self._line_height = Number(1.5)

    @property
    def line_height(self) -> Number:
        """
        Get a line-height value.

        Returns
        -------
        line_height : Number
            A line-height value.

        References
        ----------
        - Text line_height property
            - https://simon-ritchie.github.io/apysc/en/text_line_height.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=300,
        ...     stage_height=250,
        ...     stage_elem_id="stage",
        ... )

        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=250,
        ...     fill_color=ap.Colors.GRAY_AAAAAA,
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.line_height = ap.Number(2.0)
        >>> multi_line_text.line_height
        Number(2.0)
        """
        self._initialize_line_height()
        return self._line_height

    @line_height.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    def line_height(self, value: Number) -> None:
        """
        Update a line-height value.

        Parameters
        ----------
        value : Number
            A line-height value.

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=300,
        ...     stage_height=250,
        ...     stage_elem_id="stage",
        ... )

        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=250,
        ...     fill_color=ap.Colors.GRAY_AAAAAA,
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.line_height = ap.Number(2.0)
        >>> multi_line_text.line_height
        Number(2.0)
        """
        from apysc._display.css_interface import CssInterface
        from apysc._validation import display_validation

        self._initialize_line_height()
        interface: CssInterface = display_validation.validate_css_interface(
            instance=self
        )
        self._line_height = value
        interface.set_css(
            name="line-height",
            value=self._line_height.to_string(),
        )

        self._append_applying_new_attr_val_exp(
            new_attr=self._line_height, attr_name="line_height"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_height, attr_name="line_height"
        )



"""The class implementation for the `add_to_drive` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAddToDriveIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `add_to_drive` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M7.71 3.52L1.15 15l3.42 5.99 6.56-11.47-3.42-6zM13.35 15H9.73L6.3 21h8.24c-.96-1.06-1.54-2.46-1.54-4 0-.7.13-1.37.35-2zM20 16v-3h-2v3h-3v2h3v3h2v-3h3v-2h-3zm.71-4.75L15.42 2H8.58v.01l6.15 10.77C15.82 11.68 17.33 11 19 11c.59 0 1.17.09 1.71.25z"/></svg>'  # noqa



"""Class implementation for the SVG text's bold mix-in.
"""

from typing import Dict
from typing import Optional

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn


class SvgTextBoldMixIn(
    VariableNameMixIn,
    RevertMixIn,
):
    _bold: bool = False

    @property
    @add_debug_info_setting(module_name=__name__)
    def bold(self) -> Boolean:
        """
        Get a boolean whether this text is a bold style or not.

        Returns
        -------
        bold_ : Boolean
            A boolean, whether this text is a bold style or not.
        """
        from apysc._expression import expression_data_util
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="bold",
        )
        bold_: Boolean = Boolean(self._bold, variable_name_suffix=suffix)
        expression: str = (
            f'{bold_.variable_name} = {self.variable_name}.font("weight") === "bold";'
        )
        expression_data_util.append_js_expression(expression=expression)
        return bold_

    @bold.setter
    def bold(self, value: Boolean) -> None:
        """
        Set a boolean whether this text is a bold style or not.

        Parameters
        ----------
        value : Boolean
            A boolean, whether this text is a bold style or not.
        """
        from apysc._expression import expression_data_util

        self._bold = value._value
        expression: str = (
            f"if ({value.variable_name}) {{"
            f'\n  {self.variable_name}.font("weight", "bold");'
            "\n} else {"
            f'\n  {self.variable_name}.font("weight", "normal");'
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

    _bold_snapshots: Optional[Dict[str, bool]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_bold_snapshots",
            value=self._bold,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._bold = self._get_snapshot_val_if_exists(
            current_value=self._bold,
            snapshot_dict=self._bold_snapshots,
            snapshot_name=snapshot_name,
        )



"""This module is the helper module for the scale interfaces.
"""

from enum import Enum
from typing import Union

from apysc._type.dictionary import Dictionary
from apysc._type.expression_string import ExpressionString
from apysc._type.number import Number


def get_coordinate_key_for_expression(
    *, coordinate: Union[float, Number]
) -> ExpressionString:
    """
    Get a key string for the expression from the x or y coordinate.

    Parameters
    ----------
    coordinate : float or Number
        X or y coordinate.

    Returns
    -------
    key_exp_str : ExpressionString
        Key expression string.
    """
    from apysc._type.variable_name_mixin import VariableNameMixIn

    if isinstance(coordinate, VariableNameMixIn):
        coordinate_str: str = coordinate.variable_name
    else:
        coordinate_str = "{:.3f}".format(coordinate)
    key_exp_str: ExpressionString = ExpressionString(value=f"String({coordinate_str})")
    return key_exp_str


class CoordinateType(Enum):
    X = 1
    Y = 2


def get_scale_updating_expression(
    *,
    coordinate: Number,
    scale_dict: Dictionary[str, Number],
    interface_variable_name: str,
    coordinate_type: CoordinateType,
) -> str:
    """
    Get a scale updating expression string from a specified coordinate.

    Parameters
    ----------
    coordinate : Number
        X or y coordinate.
    scale_dict : Dictionary
        Scale value dictionary.
    interface_variable_name : str
        Scale interface instance variable name.
    coordinate_type : CoordinateType
        Coordinate type to identify the x or y target.

    Returns
    -------
    expression : str
        A scale updating expression string.
    """
    from apysc._display import scale_interface_helper
    from apysc._expression import expression_variables_util
    from apysc._expression import var_names
    from apysc._type import value_util

    before_value_name: str = expression_variables_util.get_next_variable_name(
        type_name=var_names.NUMBER
    )
    key_exp_str_1: ExpressionString = (
        scale_interface_helper.get_coordinate_key_for_expression(coordinate=coordinate)
    )
    key_exp_str_2: ExpressionString = (
        scale_interface_helper.get_coordinate_key_for_expression(
            coordinate=float(coordinate._value)
        )
    )
    after_value_str: str = value_util.get_value_str_for_expression(
        value=scale_dict._value[key_exp_str_2.value]
    )
    coordinate_value_str: str = value_util.get_value_str_for_expression(
        value=coordinate
    )
    scale_from_point_value_str: str = value_util.get_value_str_for_expression(
        value=scale_dict
    )
    if coordinate_type == CoordinateType.X:
        coordinate_exp: str = f"{coordinate_value_str}, 0"
        scale_resetting_exp: str = f"1 / {before_value_name}, 1"
        scale_result_exp: str = f"{after_value_str}, 1"
    else:
        coordinate_exp = f"0, {coordinate_value_str}"
        scale_resetting_exp = f"1, 1 / {before_value_name}"
        scale_result_exp = f"1, {after_value_str}"
    expression: str = (
        f"if ({key_exp_str_1.value} in "
        f"{scale_from_point_value_str}) {{"
        f"\n  var {before_value_name} = "
        f"{scale_from_point_value_str}[{key_exp_str_1.value}];"
        "\n}else {"
        f"\n  {before_value_name} = 1.0;"
        "\n}"
        f"\n{interface_variable_name}.scale("
        f"{scale_resetting_exp}, {coordinate_exp});"
        f"\n{interface_variable_name}.scale("
        f"{scale_result_exp}, {coordinate_exp});"
        f"\n{scale_from_point_value_str}[{key_exp_str_1.value}] = "
        f"{after_value_str};"
    )
    return expression
